@@ -6,10 +6,11 @@ import (
 )
 
 type Case struct {
-	Pattern string
-	Subject string
-	Match   bool
-	Opts    []opt
+	Pattern   string
+	Subject   string
+	Match     bool
+	Opts      []opt
+	MatchOpts MatchOpts
 }
 
 func (c *Case) Assert(t *testing.T) {
@@ -22,7 +23,8 @@ func (c *Case) Assert(t *testing.T) {
 	}()
 
 	p := NewWildmatch(c.Pattern, c.Opts...)
-	if p.Match(c.Subject) != c.Match {
+	if (c.MatchOpts != MatchOpts{} && p.MatchWithOpts(c.Subject, c.MatchOpts) != c.Match) ||
+		(c.MatchOpts == MatchOpts{} && p.Match(c.Subject) != c.Match) {
 		if c.Match {
 			t.Errorf("expected match: %s, %s", c.Pattern, c.Subject)
 		} else {
@@ -200,7 +202,7 @@ var Cases = []*Case{
 	{
 		Pattern: `**/bar/*`,
 		Subject: `deep/foo/bar/baz/`,
-		Match:   false,
+		Match:   true,
 	},
 	{
 		Pattern: `**/bar/**`,
@@ -217,6 +219,26 @@ var Cases = []*Case{
 		Subject: `deep/foo/bar/`,
 		Match:   true,
 	},
+	{
+		Pattern: `**/bar/**`,
+		Subject: `deep/foo/bar`,
+		Match:   false,
+	},
+	{
+		Pattern: `**/bar/**/*`,
+		Subject: `deep/foo/bar/`,
+		Match:   true,
+	},
+	{
+		Pattern: `**/bar/**/*`,
+		Subject: `deep/foo/bar`,
+		Match:   false,
+	},
+	{
+		Pattern: `**/bar/**/*`,
+		Subject: `deep/bar/bar`,
+		Match:   false,
+	},
 	{
 		Pattern: `*/bar/**`,
 		Subject: `foo/bar/baz/x`,
@@ -619,6 +641,12 @@ var Cases = []*Case{
 		Opts:    []opt{Basename},
 		Match:   true,
 	},
+	{
+		Pattern: `file.txt`,
+		Subject: `file.txt`,
+		Opts:    []opt{Basename, Contents},
+		Match:   true,
+	},
 	{
 		Pattern: `*.txt`,
 		Subject: `path/to/file.txt`,
@@ -648,6 +676,169 @@ var Cases = []*Case{
 		Opts:    []opt{Basename},
 		Match:   false,
 	},
+	{
+		Pattern: `path/`,
+		Subject: `path/to/some/intermediaries/to/file.txt`,
+		Match:   false,
+	},
+	{
+		// GitAttribute-style matching directory.
+		// false becalse gitattribute never matches directories.
+		Pattern:   `anotherfile.txt/`,
+		Subject:   `anotherfile.txt`,
+		Opts:      []opt{GitAttributes},
+		MatchOpts: MatchOpts{IsDirectory: true},
+		Match:     false,
+	},
+	{
+		// gitAttribute-style matching normal file.
+		// false as gitattribute matches ending in '/' indicate
+		// trying to match directory but gitattribute never matches directory
+		Pattern: `anotherfile1.txt/`,
+		Subject: `anotherfile1.txt`,
+		Opts:    []opt{GitAttributes},
+		Match:   false,
+	},
+	{
+		// gitignore-style matching directory.
+		Pattern:   `anotherfile2.txt/`,
+		Subject:   `anotherfile2.txt`,
+		MatchOpts: MatchOpts{IsDirectory: true},
+		Match:     true,
+	},
+	{
+		Pattern: `anotherfile3.txt/`,
+		Subject: `anotherfile3.txt`,
+		Match:   false,
+	},
+	{
+		Pattern: `anotherfile4.txt`,
+		Subject: `anotherfile4.txt/`,
+		Opts:    []opt{GitAttributes},
+		Match:   false,
+	},
+	{
+		Pattern: `**/pdfkit.frameworks/pdfkit/**`,
+		Subject: `MyFolder/libs/pdfkit.frameworks/pdfkit`,
+		Match:   false,
+	},
+	{
+		Pattern:   `foo/`,
+		Subject:   `bar/baz/foo`,
+		MatchOpts: MatchOpts{IsDirectory: true},
+		Match:     true,
+	},
+	{
+		Pattern:   `foo/`,
+		Subject:   `foo`,
+		MatchOpts: MatchOpts{IsDirectory: true},
+		Match:     true,
+	},
+	{
+		Pattern: `foo/`,
+		Subject: `foo/`,
+		Match:   true,
+	},
+	{
+		Pattern: `/foo/`,
+		Subject: `foo/`,
+		Match:   true,
+	},
+	{
+		Pattern: `big/b`,
+		Subject: `big/b/b1`,
+		Opts:    []opt{Contents},
+		Match:   true,
+	},
+	{
+		Pattern: `big`,
+		Subject: `big/b/b1`,
+		Opts:    []opt{Contents},
+		Match:   true,
+	},
+	{
+		Pattern: `b`,
+		Subject: `big/b/b1`,
+		Opts:    []opt{Contents},
+		Match:   true,
+	},
+	{
+		Pattern: `/foo/`,
+		Subject: `foo/`,
+		Opts:    []opt{Contents},
+		Match:   true,
+	},
+	{
+		Pattern: `/foo/`,
+		Subject: `foo/`,
+		Opts:    []opt{Basename, Contents},
+		Match:   true,
+	},
+	{
+		Pattern: `/foo`,
+		Subject: `foo`,
+		Match:   true,
+	},
+	{
+		Pattern: `/foo/filename.txt`,
+		Subject: `foo/filename.txt`,
+		Match:   true,
+	},
+	{
+		Pattern: `/foo/filename.txt`,
+		Subject: `bar/foo/filename.txt`,
+		Match:   false,
+	},
+	{
+		Pattern: `/foo/*.txt`,
+		Subject: `foo/filename.txt`,
+		Match:   true,
+	},
+	{
+		Pattern: `/*.txt`,
+		Subject: `foo/filename.txt`,
+		Match:   false,
+	},
+	{
+		Pattern: `/foo/*.txt`,
+		Subject: `bar/foo/filename.txt`,
+		Match:   false,
+	},
+	{
+		Pattern:   `/foo/`,
+		Subject:   `foo`,
+		MatchOpts: MatchOpts{IsDirectory: true},
+		Match:     true,
+	},
+	{
+		Pattern: `/foo/`,
+		Subject: `foo/filename.txt`,
+		Opts:    []opt{Contents},
+		Match:   true,
+	},
+	{
+		Pattern: `/foo/**`,
+		Subject: `foo/filename.txt`,
+		Match:   true,
+	},
+	{
+		Pattern: `path/`,
+		Subject: `path/to/some/intermediaries/to/file.txt`,
+		Opts:    []opt{Contents},
+		Match:   true,
+	},
+	{
+		Pattern: `to/`,
+		Subject: `path/to/some/intermediaries/to/file.txt`,
+		Opts:    []opt{Contents},
+		Match:   true,
+	},
+	{
+		Pattern: `nonexistent/`,
+		Subject: `path/to/some/intermediaries/to/file.txt`,
+		Opts:    []opt{Contents},
+		Match:   false,
+	},
 }
 
 func TestWildmatch(t *testing.T) {
